package com.tenlionsoft.aimz_k.dao

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.Query
import androidx.room.Transaction
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgCategoryBean

@Dao
interface MsgCategoryDao {


    /**
     * 获取全部信息
     *
     * @return
     */
    @Query("SELECT * FROM db_category")
    suspend fun getAllCategory(): List<MsgCategoryBean?>

    /**
     * 根据发送人ID获取  select * from db_category where loginUserId = '004611a6-4514-4e9d-bf6a-19eac8cc75cd' and senderId in ('004611a6-4514-4e9d-bf6a-19eac8cc75cd','c7b7ca3b-1828-4cdf-aa59-5e8efacd6939') and receiverId in ('c7b7ca3b-1828-4cdf-aa59-5e8efacd6939','004611a6-4514-4e9d-bf6a-19eac8cc75cd')
     */
//    @Query("SELECT * FROM db_category WHERE `loginUserId` = :userId AND (`senderId`=:receiverId AND `receiverId`=:senderId) OR (`senderId`=:senderId AND `receiverId`=:receiverId)")
//    suspend fun getCategoryBySenderId(
//        userId: String, receiverId: String, senderId: String
//    ): MsgCategoryBean?

    @Query("select * from db_category where loginUserId = :userId and senderId = :senderId  and receiverId = :receiverId")
    suspend fun getCategoryBySenderId(
        userId: String, receiverId: String, senderId: String
    ): MutableList<MsgCategoryBean>?

    /**
     * 根据ID获取消息
     *
     * @param id
     * @return
     */
    @Query("SELECT * FROM db_category WHERE id =(:id) ")
    suspend fun getCategoryById(id: String): List<MsgCategoryBean>

    /**
     * 根据信息来源查询消息
     *
     * @param from
     * @return
     */
    @Query("SELECT * FROM db_category WHERE `senderId`=(:from)")
    suspend fun getCategoryByFrom(from: String?): List<MsgCategoryBean>


    /**
     * 查询与单人的聊天记录  SELECT id FROM db_msg WHERE `from`=(:form) AND `to`=(:to) OR `from`=(:to) AND `to`=(:form) ORDER BY timestamp DESC LIMIT (((:page)-1)*(:pageSize)),(:pageSize)
     */
    @Query("SELECT * FROM db_category t1 WHERE loginUserId=:userId ORDER by t1.timestamp DESC LIMIT (((:page)-1)*(:pageSize)),(:pageSize)")
    suspend fun getCategoryPage(
        userId: String, pageSize: Int, page: Int
    ): MutableList<MsgCategoryBean>


    /**
     * 清空与某个人的聊天记录
     */
    @Query("DELETE FROM db_category WHERE loginUserId=:userId AND (senderId=:from AND receiverId=:to) OR (senderId=:to AND receiverId=:from)")
    suspend fun delChatBySenderId(userId: String, from: String, to: String)

//    //删除聊天
//    @Query("DELETE FROM db_category WHERE senderId=:senderId AND loginUserId=:userId")
//    suspend fun delChatBySenderId(userId: String, senderId: String)

    /**
     * 添加消息多个
     *
     * @param beans
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAll(vararg beans: MsgCategoryBean)

    /**
     * 批量添加
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAll(beans: List<MsgCategoryBean>)


    /**
     * 添加消息-单个
     *
     * @param bean
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertMsg(bean: MsgCategoryBean): Long

    /**
     * 清除全部聊天记录
     */
    @Query("DELETE FROM db_category")
    suspend fun delAllMsg()

    /**
     * 根据发送人的ID插入或者更新
     */
    @Transaction
    suspend fun updateOrInsert(
        loginUserId: String, receiverId: String, senderId: String, msgBean: MsgBean
    ): Long {
        //判断消息是否存在
        val bean = getCategoryBySenderId(loginUserId, receiverId, senderId)
        return if (bean == null || bean.size <= 0) {
            //需要转换成MsgCategoryBean
            insertMsg(ConvertBeanUtils.convertCategoryBean(msgBean))
        } else {
            updateCategory(
                senderId,
                receiverId,
                msgBean.messageId!!,
                msgBean.status,
                msgBean.body!!,
                msgBean.timestamp!!,
                msgBean.msgType!!,
                loginUserId
            ).toLong()
        }
    }


    //更具当前登录人消息ID更新
    @Query("UPDATE db_category SET status = :status WHERE messageId = :messageId and loginUserId = :loginUId")
    suspend fun updateStatByMessageId(status: String, messageId: String, loginUId: String): Int

    /**
     * 更新 消息体 时间戳 状态
     *   @Query("UPDATE db_msg SET status = :status WHERE messageId = :messageId")
     */
    @Query(
        "UPDATE db_category SET status = :status , body = :body , msgType = :msgType, messageId = :messageId, timestamp = :time" + " WHERE loginUserId = :loginUserId AND senderId = :senderId AND receiverId = :receiverId"
    )
    suspend fun updateCategory(
        senderId: String,
        receiverId: String,
        messageId: String,
        status: String?,
        body: String,
        time: Long,
        msgType: Int,
        loginUserId: String
    ): Int

    /**
     * 更新 头像和名称
     */
    @Query("UPDATE db_category SET userNickName=:nickName,avatar=:avatar WHERE receiverId=:userId OR senderId=:userId")
    suspend fun updateUserInfoById(nickName: String, avatar: String, userId: String)

    /**
     * 根据messageId 更新头像和名称
     */
    @Query("UPDATE db_category SET userNickName=:nickName,avatar=:avatar WHERE messageId=:messageId AND loginUserId=:loginUserId")
    suspend fun updateInfoByMessageId(
        nickName: String, avatar: String, messageId: String, loginUserId: String
    )

    @Query("UPDATE db_category set userNickName = :nickName,avatar = :avatar where messageId = :msgId and loginUserId = :logId")
    suspend fun updateInByMsgId(nickName: String, avatar: String,msgId:String,logId:String):Int
}