package com.tenlionsoft.aimz_k.page.fragments

import android.annotation.SuppressLint
import android.app.Activity
import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.FragmentMsgBinding
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.page.activity.ChatActivity
import com.tenlionsoft.aimz_k.page.activity.MainActivity
import com.tenlionsoft.aimz_k.page.activity.SearchChatMsgActivity
import com.tenlionsoft.aimz_k.services.SocketService
import com.tenlionsoft.aimz_k.viewmodel.MsgViewModel
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.AppUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.AdapterItemLongClickListener
import com.tenlionsoft.baselib.widget.LoadingDialog
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class MsgFragment : Fragment(), AdapterItemClickListener<MsgCategoryBean>,
    MainActivity.OnSocketConnectListener, AdapterItemLongClickListener<MsgCategoryBean> {
    private lateinit var mMsgBinding: FragmentMsgBinding
    private var mActivity: MainActivity? = null
    private var mLoading: LoadingDialog? = null

    companion object {
        fun newInstance() = MsgFragment()
    }

    private lateinit var viewModel: MsgViewModel


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View {
        mMsgBinding = DataBindingUtil.inflate(
            layoutInflater, R.layout.fragment_msg, container, false
        )
        //搜索
        mMsgBinding.llSearchLayout.llSearchLayout.setOnClickListener {
            mActivity?.startActivity(Intent(mActivity, SearchChatMsgActivity::class.java))
        }
        viewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return MsgViewModel(mActivity!!.application) as T
            }
        })[MsgViewModel::class.java]
        mMsgBinding.msgViewModel = viewModel
        mMsgBinding.lifecycleOwner = this
        viewModel.onItemClickListener = this//条目点击
        viewModel.onItemLongClickListener = this//长按
        initView()
        return mMsgBinding.root
    }

    /**
     * 初始化页面
     */
    private fun initView() {
        mMsgBinding.srlMsgs.setOnLoadMoreListener { viewModel.loadMore() }
        mMsgBinding.srlMsgs.setOnRefreshListener { viewModel.refresh() }

        viewModel.isHasMore.observe(viewLifecycleOwner) {
            mMsgBinding.srlMsgs.setNoMoreData(it)
            mMsgBinding.srlMsgs.finishRefresh()
            mMsgBinding.srlMsgs.finishLoadMore()
        }
        viewModel.isLogining.observe(viewLifecycleOwner) {
            if (it) {
                mMsgBinding.llSearchLayout.tvSocketStatus.text = "登陆中..."
                mMsgBinding.llSearchLayout.pbSocketLoading.visibility = View.VISIBLE
            } else {
                mMsgBinding.llSearchLayout.tvSocketStatus.visibility = View.GONE
                mMsgBinding.llSearchLayout.pbSocketLoading.visibility = View.GONE
            }
        }
        viewModel.showLoadDialog.observe(viewLifecycleOwner) {
            if (it) {
                //显示loading
                mLoading = LoadingDialog.Builder(mActivity as Activity).setCancelOutside(false)
                    .setCancelable(false).setMessage("删除中...").create()
                mLoading!!.show()
            } else {
                //隐藏loading
                mLoading?.dismiss()
                mLoading = null
            }
        }
        //登陆socket
        viewModel.doLoginWebsocket()
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is MainActivity) {
            mActivity = context
            mActivity!!.setOnSocketListener(this)
        }
    }

    //开启service
    private fun toStartService() {
        //判断是否允许
        val isRunning =
            AppUtils.isRunningService(mActivity!!, mActivity!!.packageName + ":tenlion_msg")
        if (isRunning) {
            mActivity?.sendBroadcast(Intent(ProjectConfig.A_S_RE_LOGIN))
        } else {
            mActivity?.startService(Intent(mActivity!!, SocketService::class.java))
        }
    }

    private fun isRunning(): Boolean {
        val manager = mActivity?.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        val list: List<ActivityManager.RunningServiceInfo> =
            manager.getRunningServices(Integer.MAX_VALUE)
        Log.e("MsgFragment", "isRunning:${list}")
        if (list.isEmpty()) return false
        for (info in list) {
            Log.e("MsgFragment", "isRunning:${info}")
            if ("com.tenlionsoft.aimz_k.services.SocketService" == info.javaClass.name) {
                return true
            }
        }
        return false
    }

    override fun onResume() {
        super.onResume()
        viewModel.refresh()
    }

    override fun onStart() {
        super.onStart()
        toStartService()
    }

    /**
     * 条目点击
     */
    override fun onItemClick(data: MsgCategoryBean) {
        val intent = Intent(mActivity, ChatActivity::class.java)
        Log.e("MsgFragment", "onItemClick:${data.msgType}")
        intent.putExtra("fromId", data.receiverId)
        intent.putExtra("name", data.userNickName)
        startActivity(intent)
    }

    //监听socket连接状态
    @SuppressLint("CheckResult")
    override fun onListener(status: String) {
        when (status) {
            //连接成功
            ProjectConfig.A_S_CONNECTED -> {
                viewModel.isLogining.value = false
            }
            //连接失败
            ProjectConfig.A_S_FAIL -> {
                viewModel.isLogining.value = true
            }
            //连接断开
            ProjectConfig.A_S_DISCONNECT -> {}
        }
    }

    //更新消息状态
    override fun updateStatus(messageId: String, status: String) {
        viewModel.updateMsgStatus(messageId, status)
    }

    //更新消息
    @SuppressLint("CheckResult")
    override fun updateInfo() {
        Observable.timer(1000, TimeUnit.MILLISECONDS).subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread()).subscribe {
                viewModel.refresh()
            }

    }


    //长按
    override fun onItemLongClick(type: Int, d: MsgCategoryBean) {
        //删除聊天
        viewModel.doDelChat(d)
    }

}