package com.tenlionsoft.aimz_k.viewmodel

import android.content.Context
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.tenlionsoft.aimz_k.adapter.ApplyContactAdapter
import com.tenlionsoft.aimz_k.model.ApplyContactBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.ToastUtils
import kotlinx.coroutines.launch

class ApplyContactViewModel(private val context: Context) : BaseViewModel() {
    val _applyList = MutableLiveData<List<ApplyContactBean>>()
    val adapter = ApplyContactAdapter(_applyList.value ?: emptyList(), this)
    val retrofit = RetrofitClient.getInstance(context)
    val userApi = retrofit.create(BaseApi::class.java)

    init {
        doGetApplyList()
    }

    private fun doGetApplyList() {
        viewModelScope.launch {
            try {
                val pageBean = retrofit.makeApiCall {
                    userApi.doGetMyApplyContactList()
                }
                if (pageBean.isNotEmpty()) {
                    _applyList.value = pageBean
                    adapter.setData(_applyList.value!!)
                } else {
                    ToastUtils.error("加载失败")
                }
            } catch (e: Exception) {
                e.printStackTrace()
                ExParse.parse(e)
            }
        }

    }

    fun doRefresh() {
        _applyList.value = emptyList()
        adapter.setData(_applyList.value!!)
        doGetApplyList()
    }

    //通过申请
    fun doAddContact(applyContactBean: ApplyContactBean) {
        viewModelScope.launch {
            try {
                val pageBean = retrofit.makeApiCall {
                    userApi.doPassApplyContact(applyContactBean.contactUserApplyId, "PASS")
                }
                if (pageBean.code == 200) {
                    _applyList.value?.forEach {
                        if (it.contactUserApplyId == applyContactBean.contactUserApplyId) {
                            it.status = "PASS"
                        }
                    }
                    adapter.setData(_applyList.value!!)
                } else {
                    ToastUtils.error(pageBean.msg ?: "网络错误,请稍后重试")
                }
            } catch (e: Exception) {
                e.printStackTrace()
                ExParse.parse(e)
            }
        }
    }
}