package com.tenlionsoft.aimz_k.viewmodel

import android.app.Application
import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.os.Message
import android.util.Log
import android.view.View
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.atwa.filepicker.result.FileMeta
import com.atwa.filepicker.result.ImageMeta
import com.atwa.filepicker.result.VideoMeta
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.adapter.ChatMsgAdapter
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.FileDataBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.PickerType
import com.tenlionsoft.aimz_k.model.ReceiverBean
import com.tenlionsoft.aimz_k.model.ReplyBean
import com.tenlionsoft.aimz_k.model.SenderBean
import com.tenlionsoft.aimz_k.model.UserInfoBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.contacts.toFileBody
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class ChatPageViewModel(
    private val fromId: String, private val toId: String, private val context: Application
) : BaseViewModel() {
    val txtMsg = MutableLiveData<String>("")
    val showSendBtn = MutableLiveData(false)//显示/隐藏发送按钮
    val showEmojiLayout = MutableLiveData(false)//显示/隐藏emoji
    val showReplyLayout = MutableLiveData(false)//显示/隐藏快速回复
    val showChooseLayout = MutableLiveData(false)//显示/隐藏选择
    val chooseType = MutableLiveData<PickerType>()//选择文件类型
    val scrollListToBottom = MutableLiveData<Boolean>(false)
    private val _msgList = MutableLiveData<List<MsgBean>>()
    private val retrofitClient = RetrofitClient.getInstance(context)
    private val netApi = retrofitClient.create(BaseApi::class.java)
    var adapter: ChatMsgAdapter = ChatMsgAdapter(
        _msgList.value
            ?: emptyList(), this
    )
    var onItemClickListener: AdapterItemClickListener<MsgBean>? = null
    private val mGson: Gson = Gson()
    private var msgHandlerList: ArrayList<MsgConvertBean> = arrayListOf()
    val _replyList = MutableLiveData<MutableList<ReplyBean>>()
    private var userInfo: UserInfoBean? = null
    private val handler = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: Message) {
            super.handleMessage(msg)
            val mBean: MsgConvertBean = msg.obj as MsgConvertBean
            val isExist = msgHandlerList.contains(mBean)
            if (isExist) {
                // 更新数据库 发送失败
                viewModelScope.launch {
                    withContext(Dispatchers.IO) {
                        Log.e("ChatPageViewModel", "Handle更新")
                        val mDao = DbManager.db.msgDao()
                        val cDao = DbManager.db.categoryDao()
                        mDao.updateStatus(
                            mBean.messageId, ProjectConfig.MSG_SEND_FAIL, SpUtils.getId()
                        )
                        cDao.updateStatByMessageId(
                            ProjectConfig.MSG_SEND_FAIL,
                            messageId = mBean.messageId,
                            SpUtils.getId()
                        )
                    }
                    for (item in _msgList.value!!) {
                        if (item.messageId == mBean.messageId) {
                            item.status = ProjectConfig.MSG_SEND_FAIL
                        }
                    }
                    adapter.setData(_msgList.value!!)
                }
            }
        }
    }

    init {

        viewModelScope.launch {
            val list = getList()
            val rList = getReplyList()

            if (rList.isNotEmpty()) {
                _replyList.value = rList
            } else {
                _replyList.value = arrayListOf()
            }
            _msgList.value = list
            adapter.setData(list)
            scrollListToBottom.value = true
        }
    }

    private suspend fun getReplyList(): MutableList<ReplyBean> {
        return withContext(Dispatchers.IO) {
            val rDao = DbManager.db.replayDao()
            return@withContext rDao.getAllReply()
        }
    }

    //获取列表
    private suspend fun getList(): List<MsgBean> {
        return withContext(Dispatchers.IO) {
            val msgDao = DbManager.db.msgDao()
            return@withContext msgDao.getMsgByFromOrToPage(SpUtils.getId(), fromId, toId)
        }
    }

    fun onTxtChange(s: CharSequence, start: Int, before: Int, count: Int) {
        this.txtMsg.value = s.toString()
        showSendBtn.value = s.isNotEmpty()
    }

    fun onShowReplyLayout() {
        showEmojiLayout.value = false
        showChooseLayout.value = false
        showReplyLayout.value = !showReplyLayout.value!!
    }

    /**
     * 切换emoji layout
     */
    fun onShowEmojiLayout() {
        showChooseLayout.value = false
        showReplyLayout.value = false
        showEmojiLayout.value = !showEmojiLayout.value!!
    }


    /**
     * 切换选择layout
     */
    fun onShowChooseLayout() {
        showEmojiLayout.value = false
        showReplyLayout.value = false
        showChooseLayout.value = !showChooseLayout.value!!
    }

    /**
     * 去选择图片
     */
    fun showChoosePic() {
        chooseType.value = PickerType.TYPE_PIC
    }

    /**
     * 去选择视频
     */
    fun showChooseVideo() {
        chooseType.value = PickerType.TYPE_VIDEO
    }

    /**
     * 去选择文件
     */
    fun showChooseFile() {
        chooseType.value = PickerType.TYPE_FILE
    }

    /**
     * 上传图片
     */
    fun uploadImages(list: List<ImageMeta?>) {
        if (list.isEmpty()) {
            return
        }
        viewModelScope.launch {
            showLoadDialog.value = true
            doUploadImages(list)
        }
    }


    /**
     * 上传视频
     */
    fun uploadVideo(meta: VideoMeta) {
        viewModelScope.launch {
            showLoadDialog.value = true
            doUploadVideo(meta)
        }
    }

    /**
     * 上传文件
     */
    fun uploadFiles(meta: FileMeta) {
        viewModelScope.launch {
            showLoadDialog.value = true
            doUploadFile(meta)
        }
    }

    private suspend fun doUploadVideo(meta: VideoMeta) {
        try {
            val body = meta.file!!.toFileBody("video")
            val bean = retrofitClient.makeApiCall {
                netApi.doUploadVideo(body)
            }
            if (bean.code == 200) {
                sendMedia(ProjectConfig.MSG_VIDEO, bean.data)
            }
            showLoadDialog.value = false
        } catch (e: Exception) {
            e.printStackTrace()
            showLoadDialog.value = false
            ExParse.parse(e)
        }
    }

    private suspend fun doUploadFile(meta: FileMeta) {
        try {
            val body = meta.file!!.toFileBody("file")
            val bean = retrofitClient.makeApiCall {
                netApi.doUploadFile(body)
            }
            if (bean.code == 200) {
                sendMedia(ProjectConfig.MSG_FILE, bean.data)
            }
            showLoadDialog.value = false
        } catch (e: Exception) {
            e.printStackTrace()
            showLoadDialog.value = false
            ExParse.parse(e)
        }
    }

    private var count: Int = 0
    private suspend fun doUploadImages(list: List<ImageMeta?>) {
        try {
            for (item in list) {
                if (item != null) {
                    val toFileBody = item.file!!.toFileBody("image")
                    val bean = retrofitClient.makeApiCall {
                        netApi.doUploadImage(toFileBody)
                    }
                    if (bean.code == 200) {
                        //上传成功
                        sendMedia(ProjectConfig.MSG_IMG, bean.data)
                    }
                    ++count
                }
            }
            if (count == list.size) {
                showLoadDialog.value = false
                count = 0
            }
        } catch (e: Exception) {
            e.printStackTrace()
            showLoadDialog.value = false
            count = 0
            ExParse.parse(e)
        }
    }


    //刷新
    private fun refresh() {
        viewModelScope.launch {
            val list = getList()
            _msgList.value = list
            adapter.setData(_msgList.value!!)
            scrollListToBottom.value = true
        }
    }

    //刷新页面
    private fun addBeanToView(msgBean: MsgBean) {
        if (userInfo != null) {
            msgBean.userAvatar = userInfo!!.avatar
            _msgList.value = _msgList.value?.plus(msgBean)
        } else {
            _msgList.value = _msgList.value?.plus(msgBean)
        }
        adapter.setData(_msgList.value!!)
    }

    /**
     * 发送文本信息
     */
    fun sendTxtMsg(v: View) {
        if (txtMsg.value!!.isNotEmpty()) {
            val b = ProjectConfig.MSG_TEXT.buildSendBean()
            val msgBean = ConvertBeanUtils.convertBeanToMsgBean(b)
            _msgList.value = _msgList.value?.plus(msgBean)
            adapter.setData(_msgList.value!!)
            sendHandlerToLooper(b)
            val intent = Intent(ProjectConfig.A_S_MSG_SEND)
            intent.putExtra(ProjectConfig.MSG_PUSH, b)
            v.context.sendBroadcast(intent)
            txtMsg.value = ""
            scrollListToBottom.value = true
        }
    }

    //发送 图片/视频/音频/文件
    private fun sendMedia(type: String, data: FileDataBean) {
        val intent = Intent(ProjectConfig.A_S_MSG_SEND)
        val b = buildSendMediaBean(type, data)
        val msgBean = ConvertBeanUtils.convertBeanToMsgBean(b)
        _msgList.value = _msgList.value?.plus(msgBean)
        adapter.setData(_msgList.value!!)
        sendHandlerToLooper(b)
        intent.putExtra(ProjectConfig.MSG_PUSH, b)
        context.sendBroadcast(intent)
        scrollListToBottom.value = true
    }

    //用于延迟更改消息状态
    private fun sendHandlerToLooper(b: MsgConvertBean) {
        msgHandlerList.add(b)
        val msg = Message()
        msg.what = b.messageId.toLong().toInt()
        msg.obj = b
        handler.sendMessageDelayed(msg, 8000)
    }

    //构建文字发送实体
    private fun String.buildSendBean(): MsgConvertBean {
        val bodyBean = BodyContent(content = txtMsg.value, null, null)
        val body = mGson.toJson(bodyBean)
        val messageId = TimeUtils.getNowDateMillis()
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = messageId.toString(),
            messageType = this,
            metadata = "",
            timestamp = messageId,
            sender = SenderBean(senderId = SpUtils.getId(), senderType = ""),
            receiver = ReceiverBean(receiverId = fromId, receiverType = "SINGLE_USER"),
            status = ProjectConfig.MSG_SEND_ING,
        )
    }


    //构建 图片/视频/文件发送实体
    private fun buildSendMediaBean(type: String, fileBean: FileDataBean): MsgConvertBean {
        val bodyBean = BodyContent(content = mGson.toJson(fileBean), null, null)
        val body = mGson.toJson(bodyBean)
        val messageId = TimeUtils.getNowDateMillis()
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = messageId.toString(),
            messageType = type,
            metadata = "",
            timestamp = messageId,
            sender = SenderBean(senderId = SpUtils.getId(), senderType = ""),
            receiver = ReceiverBean(receiverId = fromId, receiverType = "SINGLE_USER"),
            status = ProjectConfig.MSG_SEND_ING,
        )
    }


    //获取回复短语
    fun getReplyData() {
        viewModelScope.launch {
            val list = getReplyList()
            if (list.isNotEmpty()) {
                _replyList.value = list
            }
        }
    }

    //更新状态
    fun doUpdateMsgStatus(id: String, status: String) {
        msgHandlerList.removeIf { it.messageId == id }
        handler.removeMessages(id.toLong().toInt())
        _msgList.value?.forEach {
            if (it.messageId == id) {
                it.status = status
            }
        }
        if (_msgList.value != null) {
            adapter.setData(_msgList.value!!)
        }
    }

    //刷新
    fun doRefresh() {
        refresh()
    }
}


