package com.tenlionsoft.aimz_k.viewmodel

import android.app.Application
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.adapter.CategoryAdapter
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.model.UserInfoBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.AdapterItemLongClickListener
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class MsgViewModel(val context: Application) : BaseViewModel() {
    private val _pwdList = MutableLiveData<List<MsgCategoryBean>>()
    var adapter: CategoryAdapter = CategoryAdapter(
        _pwdList.value
            ?: emptyList(), this
    )
    var onItemClickListener: AdapterItemClickListener<MsgCategoryBean>? = null
    var onItemLongClickListener: AdapterItemLongClickListener<MsgCategoryBean>? = null
    val isLogining = MutableLiveData(false)
    private val mGson = Gson()
    private var mCurrentPage: Int = 1
    var isHasMore: MutableLiveData<Boolean> = MutableLiveData<Boolean>(false)
    private val retrofitClient = RetrofitClient.getInstance(context)
    private val baseApi = retrofitClient.create(BaseApi::class.java)
    private var userInfo: UserInfoBean? = null

    /**
     * 获取列表数据
     */
    private fun getList(isFirst: Boolean) {
        viewModelScope.launch {
            if (isFirst) {
                _pwdList.value = emptyList()
                mCurrentPage = 1
            } else {
                ++mCurrentPage
            }
            val list = getCategoryListByPage(mCurrentPage)
            isHasMore.value = list.isEmpty()
            val updateData: List<MsgCategoryBean> = _pwdList.value!! + list
            _pwdList.value = updateData
            adapter.setData(_pwdList.value!!)
        }
    }

    private suspend fun getCategoryListByPage(page: Int): List<MsgCategoryBean> {
        var tempList: List<MsgCategoryBean>
        withContext(Dispatchers.IO) {
            val cDao = DbManager.db.categoryDao()
            tempList = cDao.getCategoryPage(SpUtils.getId(), 10, page)
        }
        return tempList
    }

    /**
     * 加载更多
     */
    fun loadMore() {
        getList(false)
    }

    /**
     * 登陆websocket
     */
    fun doLoginWebsocket() {
        isLogining.value = true
    }


    /**
     * 刷新
     */
    fun refresh() {
        getList(true)
    }

    //删除聊天
    fun doDelChat(d: MsgCategoryBean) {
        viewModelScope.launch {
            showLoadDialog.value = true
            withContext(Dispatchers.IO) {
                val cDao = DbManager.db.categoryDao()
                val mDao = DbManager.db.msgDao()
                cDao.delChatBySenderId(SpUtils.getId(), d.senderId!!, d.receiverId!!)
                mDao.delChatHistory(SpUtils.getId(), d.senderId!!, d.receiverId!!)
            }
            showLoadDialog.value = false
            ToastUtils.success("删除成功")
            refresh()
        }
    }

    //更新消息状态
    fun updateMsgStatus(messageId: String, status: String) {
//        _pwdList.value?.forEach {
//            if (it.messageId == messageId) {
//                it.status = status
//            }
//        }
//        adapter.setData(_pwdList.value!!)
    }


}