package com.tenlionsoft.aimz_k.widget

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.TextView
import com.lxj.xpopup.interfaces.OnConfirmListener
import com.tenlionsoft.aimz_k.R


class ApplyPermissionDialog : Dialog {
    constructor(context: Context) : super(context)

    constructor(context: Context, themeResId: Int) : super(context, themeResId)

    constructor(
        context: Context, cancelable: Boolean, cancelListener: DialogInterface.OnCancelListener?
    ) : super(context, cancelable, cancelListener)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val window = this.window
        val displayMetrics = context.resources.displayMetrics
        window!!.setGravity(Gravity.CENTER)
        window.setWindowAnimations(com.tenlionsoft.baselib.R.style.dialog_style)
        val params = window.attributes
        params.width = (displayMetrics.widthPixels * 0.8).toInt()
        params.height = (displayMetrics.heightPixels * 0.2).toInt()
        window.attributes = params
        window.setBackgroundDrawable(ColorDrawable())

    }

    class Builder(private val context: Activity) {
        //提示信息
        private var message: String? = null

        //是否展示提示信息
        private var isShowMessage = true

        //是否按返回键取消
        private var isCancelable = true

        //是否取消
        private var isCancelOutside = false
        private var listener: OnConfirmListener? = null

        /**
         * 设置提示信息
         * @param message
         * @return
         */
        fun setMessage(message: String?): Builder {
            this.message = message
            return this
        }

        fun addConfirmListener(listener: OnConfirmListener): Builder {
            this.listener = listener
            return this
        }

        /**
         * 设置是否显示提示信息
         * @param isShowMessage
         * @return
         */
        fun setShowMessage(isShowMessage: Boolean): Builder {
            this.isShowMessage = isShowMessage
            return this
        }

        /**
         * 设置是否可以按返回键取消
         * @param isCancelable
         * @return
         */
        fun setCancelable(isCancelable: Boolean): Builder {
            this.isCancelable = isCancelable
            return this
        }

        /**
         * 设置是否可以取消
         * @param isCancelOutside
         * @return
         */
        fun setCancelOutside(isCancelOutside: Boolean): Builder {
            this.isCancelOutside = isCancelOutside
            return this
        }

        //创建Dialog
        fun create(): ApplyPermissionDialog {
            val inflater = LayoutInflater.from(context)
            val view: View = inflater.inflate(R.layout.dialog_apply_permission, null)
            //设置带自定义主题的dialog
            val dialog =
                ApplyPermissionDialog(context, com.tenlionsoft.baselib.R.style.dialog_style)
            view.findViewById<TextView>(R.id.tv_hint).text = message
                ?: "为了能正常使用,需要相关权限,请前往设置进行授权."
            view.findViewById<Button>(R.id.btn_cancel).setOnClickListener {
                dialog.dismiss()
                listener?.onCancel()
            }
            view.findViewById<Button>(R.id.btn_confirm).setOnClickListener {
                dialog.dismiss()
                listener?.onConfirm()
            }
            dialog.setContentView(view)
            dialog.setCancelable(isCancelable)
            dialog.setCanceledOnTouchOutside(isCancelOutside)
            return dialog
        }


    }

    interface OnConfirmListener {
        fun onConfirm()
        fun onCancel()
    }
}