package com.tenlionsoft.baselib.utils

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import androidx.core.content.FileProvider
import java.io.File


object FileUtils {
    private val MIME_TABLE = mapOf(
        ".doc" to "application/msword",
        ".docx" to "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
        ".xls" to "application/vnd.ms-excel",
        ".xlsx" to "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
        ".pdf" to "application/pdf",
        ".pps" to "application/vnd.ms-powerpoint",
        ".ppt" to "application/vnd.ms-powerpoint",
        ".pptx" to "application/vnd.openxmlformats-officedocument.presentationml.presentation",
        ".z" to "application/x-compress",
        ".zip" to "application/x-zip-compressed"
    )

    fun getFileProviderName(context: Context): String {
        return context.packageName + ".provider"
    }

    fun openFile(context: Context, fileName: String?) {
        val intent = Intent()
        val file = File(fileName)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK) //设置标记
        intent.setAction(Intent.ACTION_VIEW) //动作，查看
        val uri = FileProvider.getUriForFile(context, getFileProviderName(context), file)
        intent.setDataAndType(uri, getMIMEType(file)) //设置类型
        context.startActivity(intent)
    }

    fun openFile(context: Context, file: File?) {
        val intent = Intent(Intent.ACTION_VIEW)
        //intent.addCategory(Intent.CATEGORY_DEFAULT);
        val uriForFile: Uri = FileProvider.getUriForFile(
            context, getFileProviderName(context), file!!
        )
        intent.addFlags(Intent.FLAG_GRANT_PERSISTABLE_URI_PERMISSION) //给目标文件临时授权
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        intent.setDataAndType(uriForFile, getMIMEType(file))
        context.startActivity(intent)
    }

    /**
     * 获取文件类型
     */
    fun getMIMEType(file: File): String {
        var type = "*/*"
        val fName = file.name
        val dotIndex = fName.lastIndexOf(".")
        if (dotIndex < 0) {
            return type
        }
        val end = fName.substring(dotIndex, fName.length).lowercase()
        if (end.isBlank()) return type
        type = MIME_TABLE[end]
            ?: return type
        Log.e("FileUtils", "getMIMEType:${type}");
        return type
    }


}