package com.tenlionsoft.medialib.base.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.viewpager.widget.PagerAdapter
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.tenlionsoft.medialib.R
import com.tenlionsoft.medialib.base.widget.PhotoView

class PhotoAdapter(private val ctx: Context, private val imgs: MutableList<String>) :
    PagerAdapter() {

    private var mOnImgClick: OnImgClick? = null

    override fun instantiateItem(container: ViewGroup, position: Int): Any {
        val view: View = LayoutInflater.from(ctx)
            .inflate(R.layout.layout_simple_photo, container, false)
        val photoView = view.findViewById<PhotoView>(R.id.pv_view)
        val options: RequestOptions =
            RequestOptions().error(com.tenlionsoft.baselib.R.drawable.ic_img_load_err)
                .placeholder(com.tenlionsoft.baselib.R.drawable.ic_img_loading)
        Glide.with(ctx)
            .load(imgs.get(position))
            .apply(options)
            .into(photoView)
        container.addView(view)
        if (mOnImgClick != null) {
            photoView.setOnClickListener { _ -> mOnImgClick?.imgClick() }
        }
        return view
    }


    fun addImgClick(onImgClick: OnImgClick?) {
        this.mOnImgClick = onImgClick
    }

    interface OnImgClick {
        fun imgClick()
    }


    override fun getCount(): Int {
        return imgs.size
    }


    override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
        container.removeView(`object` as View)
    }

    override fun isViewFromObject(view: View, `object`: Any): Boolean {
        return view == `object`
    }
}